﻿using System;
using System.IO;
using System.Net;
using System.Xml;
using System.Linq;
using System.Text;
using System.Collections.Generic;

namespace CAPA_API_Calls
{
    /// <summary>
    /// CAPA_API contains the following functions:
    /// <para>Submit_CAPA_Job</para>
    /// <para>Check_CAPA_Job</para>
    /// <para>Abort_CAPA_Job</para>
    /// </summary>
    public class CAPA_API
    {
        private string _CAPA_Password;
        private string _Environment;
        private string _ProcessID;
        private string _ProcessPath;
        private string _CAPA_Username;
        private string _AdditionalParams;

        /// <summary>
        /// Used in: (All Functions)
        /// Environment (String) [Not-Optional] should be a valid CAPA FQDN URL
        /// </summary>
        public string Environment
        {
            get { return _Environment; }
            set { _Environment = value; }
        }

        /// <summary>
        /// Used in: Submit_CAPA_Job
        /// <para>ProcessPath (String) [Not-Optional] should be the full path to, and CAPA process name.</para>
        /// </summary>
        public string ProcessPath
        {
            get { return _ProcessPath; }
            set { _ProcessPath = value; }
        }

        /// <summary>
        /// ProcessID (String) should be the CAPA process ID.
        /// </summary>
        public string ProcessID
        {
            get { return _ProcessID; }
            set { _ProcessID = value; }
        }

        /// <summary>
        /// CAPA_Username (String) [Not-Optional] should be the AD username to use to launch the CAPA job.
        /// </summary>
        public string CAPA_Username
        {
            get { return _CAPA_Username; }
            set { _CAPA_Username = value; }
        }

        /// <summary>
        /// CAPA_Password (String) [Not-Optional] should be the AD password to use to launch the CAPA job.
        /// </summary>
        public string CAPA_Password
        {
            get { return _CAPA_Password; }
            set { _CAPA_Password = value; }
        }

        /// <summary>
        /// Used in: Submit_CAPA_Job
        /// <para>AdditionalParams (String) [Optional] should be any custom CAPA parameters you wish to pass.</para>
        /// <para>Format of the paramter should be as follows:</para>
        /// <para>&lt;itp:param name=\"param1\"&gt;Value1&lt;/itp:param&gt;&lt;itp:param name=\"param2\"&gt;Value2&lt;/itp:param&gt;</para>
        /// <para>The string will accept multiple values concatinated together.</para>
        /// </summary>
        /// <value>The element type of the array</value>
        public string AdditionalParams
        {
            get { return _AdditionalParams; }
            set { _AdditionalParams = value; }
        }

        /// <summary>Use this function to launch a CAPA job, it will return a string array. 
        /// <para>Result [0] is the status: 0 = Success, 1 = Failure.</para>
        /// <para>Result [1] is the raw SOAP result, or Error message if failure.</para>
        /// </summary>
        public string[] Submit_CAPA_Job()
        {
            try
            {
                if ((Environment != null) && (ProcessPath != null))
                {

                    if (String.IsNullOrEmpty(CAPA_Username))
                    {
                        throw new ArgumentException("CAPA_Username NullOrEmpty", "Submit_CAPA_Job");
                    }

                    if (String.IsNullOrEmpty(CAPA_Password))
                    {
                        throw new ArgumentException("CAPA_Password NullOrEmpty", "Submit_CAPA_Job");
                    }

                    // Prepare SOAP URL
                    String SoapURL = Environment;
                    String soapString = @"<soapenv:Envelope xmlns:soapenv=""http://schemas.xmlsoap.org/soap/envelope/"" xmlns:itp=""http://www.ca.com/itpam"">
                                   <soapenv:Header/>
                                   <soapenv:Body>
                                      <itp:executeProcess>
                                         <itp:flow>
                                            <itp:name>" + ProcessPath + @"</itp:name>
                                            <itp:action>start</itp:action>
                                            <itp:auth>
                                               <itp:user>" + CAPA_Username + @"</itp:user>
                                               <itp:password>" + CAPA_Password + @"</itp:password>
                                            </itp:auth>
                                            <itp:params>
                                                " + AdditionalParams + @"
                                            </itp:params>
                                         </itp:flow>
                                      </itp:executeProcess>
                                   </soapenv:Body>
                                </soapenv:Envelope>
                                ";

                    try
                    {
                        // Execute SOAP Request
                        var soapWebRequest = System.Net.WebRequest.Create(SoapURL);
                        soapWebRequest.Headers.Add("SOAPAction", "\"ExecuteC2OFlow\"");
                        soapWebRequest.ContentType = "text/xml;charset=\"utf-8\"";
                        soapWebRequest.Method = "POST";

                        XmlDocument soapEnvelop = new XmlDocument();
                        soapEnvelop.LoadXml(soapString);

                        var requestStream = soapWebRequest.GetRequestStream();
                        soapEnvelop.Save(requestStream);
                        requestStream.Close();

                        var resp = soapWebRequest.GetResponse();
                        var responseStream = resp.GetResponseStream();
                        System.IO.StreamReader soapReader = new System.IO.StreamReader(responseStream);

                        string[] FunctionReply = new string[2];
                        FunctionReply[0] = "0";
                        FunctionReply[1] = soapReader.ReadToEnd();
                        return FunctionReply;
                    }
                    catch (WebException ex)
                    {
                        using (var stream = ex.Response.GetResponseStream())
                        using (var reader = new StreamReader(stream))
                        {
                            string[] FunctionReply = new string[2];
                            FunctionReply[0] = "1";
                            FunctionReply[1] = reader.ReadToEnd();
                            return FunctionReply;
                        }
                    }
                    catch (Exception ex)
                    {
                        string[] FunctionReply = new string[2];
                        FunctionReply[0] = "1";
                        FunctionReply[1] = ex.ToString();
                        return FunctionReply;
                    }
                }
                else
                {
                    string ErrorValue = "";
                    if (Environment == null) { ErrorValue += "Environment variable was not supplied. "; }
                    if (ProcessPath == null) { ErrorValue += "ProcessPath variable was not supplied. "; }

                    string[] FunctionReply = new string[2];
                    FunctionReply[0] = "1";
                    FunctionReply[1] = "Missing parameters: " + ErrorValue;
                    return FunctionReply;
                }
            }
            catch (Exception exp)
            {
                throw exp;
            }
        }

        /// <summary>Use this function to abort a running CAPA job, it will return a string array. 
        /// <para>Result [0] is the status: 0 = Success, 1 = Failure.</para>
        /// <para>Result [1] is the raw SOAP result, or Error message if failure.</para>
        /// <para> </para>
        /// <para>**Before calling make sure to set the following variables**</para>
        /// <para>Environment, ProcessID</para>
        /// </summary>
        public string[] Abort_CAPA_Job()
        {
            if ((Environment != null) && (ProcessID != null))
            {

                if (String.IsNullOrEmpty(CAPA_Username))
                {
                    throw new ArgumentException("CAPA_Username NullOrEmpty", "Abort_CAPA_Job");
                }

                if (String.IsNullOrEmpty(CAPA_Password))
                {
                    throw new ArgumentException("CAPA_Password NullOrEmpty", "Abort_CAPA_Job");
                }

                // Prepare SOAP URL
                String SoapURL = Environment;
                String soapString = @"<soapenv:Envelope xmlns:soapenv=""http://schemas.xmlsoap.org/soap/envelope/"" xmlns:itp=""http://www.ca.com/itpam"">
                                   <soapenv:Header/>
                                   <soapenv:Body>
                                      <itp:controlProcess>
                                          <itp:ProcessID>" + ProcessID + @"</itp:ProcessID>
                                          <itp:action>abort</itp:action>
                                          <itp:auth>
                                             <itp:user>" + CAPA_Username + @"</itp:user>
                                             <itp:password>" + CAPA_Password + @"</itp:password>
                                          </itp:auth>
                                      </itp:controlProcess>
                                   </soapenv:Body>
                                </soapenv:Envelope>
                                ";

                try
                {
                    // Execute SOAP Request
                    var soapWebRequest = System.Net.WebRequest.Create(SoapURL);
                    soapWebRequest.Headers.Add("SOAPAction", "\"controlProcess\"");
                    soapWebRequest.ContentType = "text/xml;charset=\"utf-8\"";
                    soapWebRequest.Method = "POST";

                    XmlDocument soapEnvelop = new XmlDocument();
                    soapEnvelop.LoadXml(soapString);

                    var requestStream = soapWebRequest.GetRequestStream();
                    soapEnvelop.Save(requestStream);
                    requestStream.Close();

                    var resp = soapWebRequest.GetResponse();
                    var responseStream = resp.GetResponseStream();
                    System.IO.StreamReader soapReader = new System.IO.StreamReader(responseStream);

                    string[] FunctionReply = new string[2];
                    FunctionReply[0] = "0";
                    FunctionReply[1] = soapReader.ReadToEnd();
                    return FunctionReply;
                }
                catch (WebException ex)
                {
                    using (var stream = ex.Response.GetResponseStream())
                    using (var reader = new StreamReader(stream))
                    {
                        string[] FunctionReply = new string[2];
                        FunctionReply[0] = "1";
                        FunctionReply[1] = reader.ReadToEnd();
                        return FunctionReply;
                    }
                }
                catch (Exception ex)
                {
                    string[] FunctionReply = new string[2];
                    FunctionReply[0] = "1";
                    FunctionReply[1] = ex.ToString();
                    return FunctionReply;
                }
            }
            else
            {
                string ErrorValue = "";
                if (Environment == null) { ErrorValue += "Environment variable was not supplied. "; }
                if (ProcessPath == null) { ErrorValue += "ProcessID variable was not supplied. "; }

                string[] FunctionReply = new string[2];
                FunctionReply[0] = "1";
                FunctionReply[1] = "Missing parameters: " + ErrorValue;
                return FunctionReply;
            }
        }

        /// <summary>Use this function to check the status of a CAPA job, it will return a string array. 
        /// <para>Result [0] is the status: 0 = Success, 1 = Failure.</para>
        /// <para>Result [1] is the raw SOAP result, or Error message if failure.</para>
        /// <para> </para>
        /// <para>**Before calling make sure to set the following variables**</para>
        /// <para>Environment, ProcessID</para>
        /// </summary>
        public string[] Check_CAPA_Job()
        {
            if ((Environment != null) && (ProcessID != null))
            {

                if (String.IsNullOrEmpty(CAPA_Username))
                {
                    throw new ArgumentException("CAPA_Username NullOrEmpty", "Check_CAPA_Job");
                }

                if (String.IsNullOrEmpty(CAPA_Password))
                {
                    throw new ArgumentException("CAPA_Password NullOrEmpty", "Check_CAPA_Job");
                }

                // Prepare SOAP URL
                String SoapURL = Environment;
                String soapString = @"<soapenv:Envelope xmlns:soapenv=""http://schemas.xmlsoap.org/soap/envelope/"" xmlns:itp=""http://www.ca.com/itpam"">
                                   <soapenv:Header/>
                                   <soapenv:Body>
                                      <itp:getProcessStatus>
                                        <itp:flow>
                                          <itp:auth>
                                             <itp:user>" + CAPA_Username + @"</itp:user>
                                             <itp:password>" + CAPA_Password + @"</itp:password>
                                          </itp:auth>
                                          <itp:action>check</itp:action>
                                          <itp:ROID>" + ProcessID + @"</itp:ROID>
                                        </itp:flow>
                                      </itp:getProcessStatus>
                                   </soapenv:Body>
                                </soapenv:Envelope>
                                ";
                try
                {
                    // Execute SOAP Request
                    var soapWebRequest = System.Net.WebRequest.Create(SoapURL);
                    soapWebRequest.Headers.Add("SOAPAction", "\"GetFlowState\"");
                    soapWebRequest.ContentType = "text/xml;charset=\"utf-8\"";
                    soapWebRequest.Method = "POST";

                    XmlDocument soapEnvelop = new XmlDocument();
                    soapEnvelop.LoadXml(soapString);

                    var requestStream = soapWebRequest.GetRequestStream();
                    soapEnvelop.Save(requestStream);
                    requestStream.Close();

                    var resp = soapWebRequest.GetResponse();
                    var responseStream = resp.GetResponseStream();
                    System.IO.StreamReader soapReader = new System.IO.StreamReader(responseStream);

                    string[] FunctionReply = new string[2];
                    FunctionReply[0] = "0";
                    FunctionReply[1] = soapReader.ReadToEnd();
                    FunctionReply[1] += "CAPAU: " + CAPA_Username + " CAPAP: " + CAPA_Password + " Env: " + Environment + " ID:" + ProcessID;
                    return FunctionReply;
                }
                catch (WebException ex)
                {
                    using (var stream = ex.Response.GetResponseStream())
                    using (var reader = new StreamReader(stream))
                    {
                        string[] FunctionReply = new string[2];
                        FunctionReply[0] = "1";
                        FunctionReply[1] = reader.ReadToEnd();
                        return FunctionReply;
                    }
                }
                catch (Exception ex)
                {
                    string[] FunctionReply = new string[2];
                    FunctionReply[0] = "1";
                    FunctionReply[1] = ex.ToString();
                    return FunctionReply;
                }
            }
            else
            {
                string ErrorValue = "";
                if (Environment == null) { ErrorValue += "Environment variable was not supplied. "; }
                if (ProcessPath == null) { ErrorValue += "ProcessID variable was not supplied. "; }

                string[] FunctionReply = new string[2];
                FunctionReply[0] = "1";
                FunctionReply[1] = "Missing parameters: " + ErrorValue;
                return FunctionReply;
            }
        }
    }
}